/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.inspection.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dayu.daf.core.log.annotation.ApiLog;
import com.xkcoding.http.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import lombok.SneakyThrows;
import org.apache.commons.codec.Charsets;
import org.energy.modules.inspection.excel.InspectionPlanExcel;
import org.energy.modules.system.util.DataUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.energy.modules.inspection.entity.InspectionPlan;
import org.energy.modules.inspection.vo.InspectionPlanVO;
import org.energy.modules.inspection.service.IInspectionPlanService;
import com.dayu.daf.core.boot.ctrl.DafController;
import springfox.documentation.annotations.ApiIgnore;

import java.net.URLEncoder;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 巡检计划 控制器
 *
 * @author Daf
 * @since 2024-07-12
 */
@RestController
@AllArgsConstructor
@RequestMapping("/inspection/inspectionplan")
@Api(value = "巡检计划", tags = "巡检计划接口")
public class InspectionPlanController extends DafController {

	private IInspectionPlanService inspectionPlanService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入inspectionPlan")
	public R<InspectionPlan> detail(InspectionPlan inspectionPlan) {
		InspectionPlan detail = inspectionPlanService.getOne(Condition.getQueryWrapper(inspectionPlan));
		return R.data(detail);
	}

	/**
	 * 分页 巡检计划
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入inspectionPlan")
	public R<IPage<InspectionPlan>> list(InspectionPlan inspectionPlan, Query query) {
		QueryWrapper<InspectionPlan> qw = new QueryWrapper<>();
		qw.orderByAsc("plan_no");
		if (StringUtil.isNotEmpty(inspectionPlan.getPlanNo())) {
			qw.lambda().like(InspectionPlan::getPlanNo, inspectionPlan.getPlanNo());
		}
		if (StringUtil.isNotEmpty(inspectionPlan.getPlanName())) {
			qw.lambda().like(InspectionPlan::getPlanName, inspectionPlan.getPlanName());
		}
		if (StringUtil.isNotEmpty(inspectionPlan.getPeriodType())) {
			qw.lambda().like(InspectionPlan::getPeriodType, inspectionPlan.getPeriodType());
		}
		if (null != inspectionPlan.getPlanStatus()) {
			qw.lambda().eq(InspectionPlan::getPlanStatus, inspectionPlan.getPlanStatus());
		}
		IPage<InspectionPlan> pages = inspectionPlanService.page(Condition.getPage(query), qw);
		return R.data(pages);
	}

	/**
	 * 自定义分页 巡检计划
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入inspectionPlan")
	public R<IPage<InspectionPlanVO>> page(InspectionPlanVO inspectionPlan, Query query) {
		IPage<InspectionPlanVO> pages = inspectionPlanService.selectInspectionPlanPage(Condition.getPage(query), inspectionPlan);
		return R.data(pages);
	}

	/**
	 * 新增 巡检计划
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入inspectionPlan")
	public R save(@Valid @RequestBody InspectionPlan inspectionPlan) {
		return R.status(inspectionPlanService.save(inspectionPlan));
	}

	/**
	 * 修改 巡检计划
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入inspectionPlan")
	public R update(@Valid @RequestBody InspectionPlan inspectionPlan) {
		return R.status(inspectionPlanService.updateById(inspectionPlan));
	}

	/**
	 * 新增或修改 巡检计划
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入inspectionPlan")
	public R submit(@Valid @RequestBody InspectionPlan inspectionPlan) {
		inspectionPlan.setPlanStatus(1);
		String date = LocalDate.now().toString().replace("-","");
		String maxNo = inspectionPlanService.getMaxNo(date);
		int number;
		if (StringUtil.isNotEmpty(maxNo)){
			number = Integer.parseInt(maxNo.substring(9));
			number++;
		}else{
			number = 1;
		}
		String numFormat = String.format("%04d", number);
		String no = "J" + date + numFormat;
		inspectionPlan.setPlanNo(no);
		return R.status(inspectionPlanService.saveOrUpdate(inspectionPlan));
	}


	/**
	 * 删除 巡检计划
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(inspectionPlanService.deleteLogic(Func.toLongList(ids)));
	}


	/**
	 * 生效
	 */
	@PostMapping("/takeEffect")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "生效", notes = "传入ids")
	public R takeEffect(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.updateInspectionPlan(1, "", ids);
	}

	/**
	 * 失效
	 */
	@PostMapping("/lapse")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "失效", notes = "传入ids")
	public R lapse(@ApiParam(value = "主键集合", required = true) @RequestParam String ids, @ApiParam(value = "取消原因", required = true) @RequestParam String reason) {
		return this.updateInspectionPlan(0, reason, ids);
	}

	/**
	 * 生效OR失效
	 */
	public R updateInspectionPlan(Integer status, String reason, String ids) {
		List<InspectionPlan> list = new ArrayList<>();
		for (String id : ids.split(",")) {
			InspectionPlan inspectionPlan = new InspectionPlan();
			inspectionPlan.setId(Long.parseLong(id));
			inspectionPlan.setCancelReason(reason);
			inspectionPlan.setPlanStatus(status);
			list.add(inspectionPlan);
		}
		return R.status(inspectionPlanService.updateBatchById(list));
	}


	/**
	 * 导出
	 */
	@SneakyThrows
	@GetMapping("export")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "导出", notes = "传入")
	@ApiLog
	public void exportInspectionRoute(@ApiIgnore @RequestParam Map<String, Object> entity, HttpServletResponse response) {
		entity.remove("daf-auth");
		InspectionPlan vo = DataUtils.mapToEntity(entity, InspectionPlan::new);
		List<InspectionPlanExcel> list = inspectionPlanService.export(vo);
		response.setContentType("application/vnd.ms-excel");
		response.setCharacterEncoding(Charsets.UTF_8.name());
		String fileName = URLEncoder.encode("巡检计划数据导出", Charsets.UTF_8.name());
		response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
		EasyExcel.write(response.getOutputStream(), InspectionPlanExcel.class).sheet("巡检计划").doWrite(list);
	}

	/**
	 * 获取路线编号
	 */
	@GetMapping("/getPlanList")
	@ApiOperationSupport(order = 11)
	@ApiOperation(value = "获取路线编号", notes = "获取路线编号")
	public R<List<InspectionPlan>> getPlanList() {
		List<InspectionPlan> routeList = inspectionPlanService.getPlanList();
		return R.data(routeList);
	}

	/**
	 * 获取路线编号
	 */
	@GetMapping("/getDetail")
	@ApiOperationSupport(order = 12)
	@ApiOperation(value = "获取路线信息", notes = "获取路线信息")
	public R<InspectionPlan> getDetail(String planNo) {
		InspectionPlan detail = inspectionPlanService.getDetail(planNo);
		return R.data(detail);
	}
}
