/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.inspection.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dayu.daf.core.log.annotation.ApiLog;
import com.xkcoding.http.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import lombok.SneakyThrows;
import org.apache.commons.codec.Charsets;
import org.energy.modules.inspection.excel.InspectionRouteExcel;
import org.energy.modules.system.util.DataUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.energy.modules.inspection.entity.InspectionRoute;
import org.energy.modules.inspection.vo.InspectionRouteVO;
import org.energy.modules.inspection.service.IInspectionRouteService;
import com.dayu.daf.core.boot.ctrl.DafController;
import springfox.documentation.annotations.ApiIgnore;

import java.net.URLEncoder;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 巡检路线 控制器
 *
 * @author Daf
 * @since 2024-07-10
 */
@RestController
@AllArgsConstructor
@RequestMapping("/inspection/inspectionRoute")
@Api(value = "巡检路线", tags = "巡检路线接口")
public class InspectionRouteController extends DafController {

	private IInspectionRouteService inspectionRouteService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入inspectionRoute")
	public R<InspectionRoute> detail(InspectionRoute inspectionRoute) {
		InspectionRoute detail = inspectionRouteService.getOne(Condition.getQueryWrapper(inspectionRoute));
		return R.data(detail);
	}

	/**
	 * 分页 巡检路线
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入inspectionRoute")
	public R<IPage<InspectionRoute>> list(InspectionRoute inspectionRoute, Query query) {
		QueryWrapper<InspectionRoute> qw = new QueryWrapper<>();
		qw.orderByAsc("route_no");
		if (StringUtil.isNotEmpty(inspectionRoute.getRouteNo())) {
			qw.lambda().like(InspectionRoute::getRouteNo, inspectionRoute.getRouteNo());
		}
		if (StringUtil.isNotEmpty(inspectionRoute.getRouteName())) {
			qw.lambda().like(InspectionRoute::getRouteName, inspectionRoute.getRouteName());
		}
		if (StringUtil.isNotEmpty(inspectionRoute.getType())) {
			qw.lambda().like(InspectionRoute::getType, inspectionRoute.getType());
		}
		if (null != inspectionRoute.getIstatus()) {
			qw.lambda().eq(InspectionRoute::getIstatus, inspectionRoute.getIstatus());
		}
		IPage<InspectionRoute> pages = inspectionRouteService.page(Condition.getPage(query), qw);
		return R.data(pages);
	}

	/**
	 * 自定义分页 巡检路线
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入inspectionRoute")
	public R<IPage<InspectionRouteVO>> page(InspectionRouteVO inspectionRoute, Query query) {
		IPage<InspectionRouteVO> pages = inspectionRouteService.selectInspectionRoutePage(Condition.getPage(query), inspectionRoute);
		return R.data(pages);
	}

	/**
	 * 新增 巡检路线
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入inspectionRoute")
	public R save(@Valid @RequestBody InspectionRoute inspectionRoute) {
		return R.status(inspectionRouteService.save(inspectionRoute));
	}

	/**
	 * 修改 巡检路线
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入inspectionRoute")
	public R update(@Valid @RequestBody InspectionRoute inspectionRoute) {
		return R.status(inspectionRouteService.updateById(inspectionRoute));
	}

	/**
	 * 新增或修改 巡检路线
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入inspectionRoute")
	public R submit(@Valid @RequestBody InspectionRoute inspectionRoute) {
		inspectionRoute.setIstatus(1);
		String date = LocalDate.now().toString().replace("-","");
		String maxNo = inspectionRouteService.getMaxNo(date);
		int number;
		if (StringUtil.isNotEmpty(maxNo)){
			number = Integer.parseInt(maxNo.substring(9));
			number++;
		}else{
			number = 1;
		}
		String numFormat = String.format("%04d", number);
		String no = "L" + date + numFormat;
		inspectionRoute.setRouteNo(no);
		return R.status(inspectionRouteService.saveOrUpdate(inspectionRoute));
	}


	/**
	 * 删除 巡检路线
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(inspectionRouteService.deleteLogic(Func.toLongList(ids)));
	}



	/**
	 * 生效
	 */
	@PostMapping("/takeEffect")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "生效", notes = "传入ids")
	public R takeEffect(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.updateInspectionRoute(1, "", ids);
	}

	/**
	 * 失效
	 */
	@PostMapping("/lapse")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "失效", notes = "传入ids")
	public R lapse(@ApiParam(value = "主键集合", required = true) @RequestParam String ids, @ApiParam(value = "取消原因", required = true) @RequestParam String reason) {
		return this.updateInspectionRoute(0, reason, ids);
	}

	/**
	 * 生效OR失效
	 */
	public R updateInspectionRoute(Integer status, String reason, String ids) {
		List<InspectionRoute> list = new ArrayList<>();
		for (String id : ids.split(",")) {
			InspectionRoute inspectionRoute = new InspectionRoute();
			inspectionRoute.setId(Long.parseLong(id));
			inspectionRoute.setCancelReason(reason);
			inspectionRoute.setIstatus(status);
			list.add(inspectionRoute);
		}
		return R.status(inspectionRouteService.updateBatchById(list));
	}


	/**
	 * 导出
	 */
	@SneakyThrows
	@GetMapping("export")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "导出", notes = "传入")
	@ApiLog
	public void exportInspectionRoute(@ApiIgnore @RequestParam Map<String, Object> entity, HttpServletResponse response) {
		entity.remove("daf-auth");
		InspectionRoute vo = DataUtils.mapToEntity(entity, InspectionRoute::new);
		List<InspectionRouteExcel> list = inspectionRouteService.export(vo);
		response.setContentType("application/vnd.ms-excel");
		response.setCharacterEncoding(Charsets.UTF_8.name());
		String fileName = URLEncoder.encode("巡检路线数据导出", Charsets.UTF_8.name());
		response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
		EasyExcel.write(response.getOutputStream(), InspectionRouteExcel.class).sheet("巡检路线").doWrite(list);
	}

	/**
	 * 获取路线编号
	 */
	@GetMapping("/getRouteList")
	@ApiOperationSupport(order = 11)
	@ApiOperation(value = "获取路线编号", notes = "获取路线编号")
	public R<List<InspectionRoute>> getRouteList() {
		List<InspectionRoute> routeList = inspectionRouteService.getRouteList();
		return R.data(routeList);
	}

	/**
	 * 获取路线编号
	 */
	@GetMapping("/getDetailList")
	@ApiOperationSupport(order = 12)
	@ApiOperation(value = "获取路线信息", notes = "获取路线信息")
	public R<InspectionRoute> getDetailList(String routeNo) {
		InspectionRoute DetailList = inspectionRouteService.getDetailList(routeNo);
		return R.data(DetailList);
	}

}
