/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.release.controller;

import com.xkcoding.http.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import org.energy.modules.smart.entity.OperationTicket;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.energy.modules.release.entity.ProductionInformation;
import org.energy.modules.release.vo.ProductionInformationVO;
import org.energy.modules.release.service.IProductionInformationService;
import com.dayu.daf.core.boot.ctrl.DafController;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * 生产信息发布 控制器
 *
 * @author Daf
 * @since 2024-07-15
 */
@RestController
@AllArgsConstructor
@RequestMapping("/release/productioninformation")
@Api(value = "生产信息发布", tags = "生产信息发布接口")
public class ProductionInformationController extends DafController {

	private IProductionInformationService productionInformationService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入productionInformation")
	public R<ProductionInformation> detail(ProductionInformation productionInformation) {
		ProductionInformation detail = productionInformationService.getOne(Condition.getQueryWrapper(productionInformation));
		return R.data(detail);
	}

	/**
	 * 分页 生产信息发布
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入productionInformation")
	public R<IPage<ProductionInformation>> list(ProductionInformation productionInformation, Query query) {
		IPage<ProductionInformation> pages = productionInformationService.page(Condition.getPage(query), Condition.getQueryWrapper(productionInformation));
		return R.data(pages);
	}

	/**
	 * 自定义分页 生产信息发布
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入productionInformation")
	public R<IPage<ProductionInformationVO>> page(ProductionInformationVO productionInformation, Query query) {
		IPage<ProductionInformationVO> pages = productionInformationService.selectProductionInformationPage(Condition.getPage(query), productionInformation);
		return R.data(pages);
	}

	/**
	 * 新增 生产信息发布
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入productionInformation")
	public R save(@Valid @RequestBody ProductionInformation productionInformation) {
		return R.status(productionInformationService.save(productionInformation));
	}

	/**
	 * 修改 生产信息发布
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入productionInformation")
	public R update(@Valid @RequestBody ProductionInformation productionInformation) {
		return R.status(productionInformationService.updateById(productionInformation));
	}

	/**
	 * 新增或修改 生产信息发布
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入productionInformation")
	public R submit(@Valid @RequestBody ProductionInformation productionInformation) {
		LocalDate currentDate = LocalDate.now();
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd");
		String formattedDate = currentDate.format(formatter);
		//获取最大消息编码
		String maxMessageCd = productionInformationService.getMaxMessageCd("'"+formattedDate+"%'");
		int number;
		if (StringUtil.isNotEmpty(maxMessageCd)){
			number = Integer.parseInt(maxMessageCd.substring(maxMessageCd.length()-4));
			number++;
		}else{
			number = 1;
		}
		String numFormat = String.format("%04d", number);
		productionInformation.setMessageEncoding(formattedDate+numFormat);

		return R.status(productionInformationService.saveOrUpdate(productionInformation));
	}


	/**
	 * 删除 生产信息发布
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(productionInformationService.deleteLogic(Func.toLongList(ids)));
	}
	/**
	 * 审核 生产信息发布
	 */
	@PostMapping("/auditing")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R auditing(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.approve(2,ids);
	}

	/**
	 * 驳回 生产信息发布
	 */
	@PostMapping("/reject")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R reject(@ApiParam(value = "主键集合", required = true) @RequestParam String ids)
	{
		return this.approve(3,ids);
	}

	/**
	 * 驳回OR审核 生产信息发布
	 */
	public R approve(Integer status, String ids) {
		List<ProductionInformation> list = new ArrayList<>();
		for (String id : ids.split(",")) {
			ProductionInformation productionInformation = new ProductionInformation();
			productionInformation.setId(Long.parseLong(id));
			productionInformation.setCheckstatus(status);
			list.add(productionInformation);
		}
		return R.status(productionInformationService.updateBatchById(list));
	}

	/**
	 * 停用 生产信息发布
	 */
	@PostMapping("/stop")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R stop(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.changeRelease(2,ids);
	}

	/**
	 * 发布 生产信息发布
	 */
	@PostMapping("/release")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R release(@ApiParam(value = "主键集合", required = true) @RequestParam String ids)
	{
		return this.changeRelease(1,ids);
	}

	/**
	 * 发布R停用 生产信息发布
	 */
	public R changeRelease(Integer discontinue, String ids) {
		List<ProductionInformation> list = new ArrayList<>();
		for (String id : ids.split(",")) {
			ProductionInformation productionInformation = new ProductionInformation();
			productionInformation.setId(Long.parseLong(id));
			productionInformation.setIsDiscontinue(discontinue);
			list.add(productionInformation);
		}
		return R.status(productionInformationService.updateBatchById(list));
	}

}
