/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.spares.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dayu.daf.core.log.annotation.ApiLog;
import com.dayu.daf.core.tool.constant.DafConstant;
import com.xkcoding.http.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import lombok.SneakyThrows;
import org.apache.commons.codec.Charsets;
import org.energy.modules.inspection.entity.InspectionRoute;
import org.energy.modules.inspection.excel.InspectionRouteExcel;
import org.energy.modules.leger.entity.EquipmentLedger;
import org.energy.modules.spares.entity.Inbound;
import org.energy.modules.spares.entity.Outbound;
import org.energy.modules.spares.excel.MaterialExcel;
import org.energy.modules.spares.service.IInboundService;
import org.energy.modules.spares.service.IOutboundService;
import org.energy.modules.system.util.DataUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.energy.modules.spares.entity.Material;
import org.energy.modules.spares.vo.MaterialVO;
import org.energy.modules.spares.service.IMaterialService;
import com.dayu.daf.core.boot.ctrl.DafController;
import springfox.documentation.annotations.ApiIgnore;

import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * 物资 控制器
 *
 * @author Daf
 * @since 2024-07-15
 */
@RestController
@AllArgsConstructor
@RequestMapping("/spares/material")
@Api(value = "物资", tags = "物资接口")
public class MaterialController extends DafController {

	private IMaterialService materialService;
	private IInboundService inboundService;
	private IOutboundService outboundService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入material")
	public R<Material> detail(Material material) {
		Material detail = materialService.getOne(Condition.getQueryWrapper(material));
		return R.data(detail);
	}

	/**
	 * 分页 物资
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入material")
	public R<IPage<Material>> list(Material material, Query query) {
		QueryWrapper<Material> qw = new QueryWrapper<>();
		qw.orderByAsc("material_no");
		// 物资编号
		if (StringUtil.isNotEmpty(material.getMaterialNo())) {
			qw.lambda().like(Material::getMaterialNo, material.getMaterialNo());
		}

		// 物资名称
		if (StringUtil.isNotEmpty(material.getMaterialName())) {
			qw.lambda().like(Material::getMaterialName, material.getMaterialName());
		}

		// 规格型号
		if (StringUtil.isNotEmpty(material.getModel())) {
			qw.lambda().like(Material::getModel, material.getModel());
		}

		// 场站
		if (null != material.getStation()) {
			qw.lambda().eq(Material::getStation, material.getStation());
		}

		// 库存数量
		if (null != material.getInventoryCount()) {
			qw.lambda().eq(Material::getInventoryCount, material.getInventoryCount());
		}

		IPage<Material> pages = materialService.page(Condition.getPage(query), qw);
		return R.data(pages);
	}

	/**
	 * 自定义分页 物资
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入material")
	public R<IPage<MaterialVO>> page(MaterialVO material, Query query) {
		IPage<MaterialVO> pages = materialService.selectMaterialPage(Condition.getPage(query), material);
		return R.data(pages);
	}

	/**
	 * 新增 物资
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入material")
	public R save(@Valid @RequestBody Material material) {
		return R.status(materialService.save(material));
	}

	/**
	 * 修改 物资
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入material")
	public R update(@Valid @RequestBody Material material) {
		return R.status(materialService.updateById(material));
	}

	/**
	 * 新增或修改 物资
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入material")
	public R submit(@Valid @RequestBody Material material) {
		material.setIstatus(2L);
		String maxNo = materialService.getMaxNo();
		int number;
		if (StringUtil.isNotEmpty(maxNo)){
			number = Integer.parseInt(maxNo.substring(2));
			number++;
		}else{
			number = 1;
		}
		String numFormat = String.format("%06d", number);
		String no = "WZ" + numFormat;
		material.setMaterialNo(no);
		return R.status(materialService.saveOrUpdate(material));
	}


	/**
	 * 删除 物资
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		for (String id : ids.split(",")) {
			// 获取物资编码
			Material sel = new Material();
			sel.setId(Long.parseLong(id));
			Material material = materialService.getOne(Condition.getQueryWrapper(sel));
			// 通过物资编码删除入库数据
			QueryWrapper<Inbound> qw1 = new QueryWrapper<>();
			qw1.lambda().eq(Inbound::getMaterialNo, material.getMaterialNo());
			qw1.lambda().eq(Inbound::getIsDeleted, DafConstant.DB_NOT_DELETED);
			inboundService.remove(qw1);
			// 通过物资编码删除出库数据
			QueryWrapper<Outbound> qw2 = new QueryWrapper<>();
			qw2.lambda().eq(Outbound::getMaterialNo, material.getMaterialNo());
			qw2.lambda().eq(Outbound::getIsDeleted, DafConstant.DB_NOT_DELETED);
			outboundService.remove(qw2);
		}
		return R.status(materialService.deleteLogic(Func.toLongList(ids)));
	}

	/**
	 * 获取物资编号
	 */
	@GetMapping("/getNoList")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "获取物资编号", notes = "获取物资编号")
	public R<List<Material>> getNoList() {
		List<Material> noList = materialService.getNoList();
		return R.data(noList);
	}

	/**
	 * 获取物资信息
	 */
	@GetMapping("/getDetailList")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "获取物资信息", notes = "获取物资信息")
	public R<MaterialVO> getDetailList(String materialNo) {
		MaterialVO DetailList = materialService.getDetailList(materialNo);
		return R.data(DetailList);
	}

	/**
	 * 导出
	 */
	@SneakyThrows
	@GetMapping("export")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "导出", notes = "传入")
	@ApiLog
	public void exportMaterial(@ApiIgnore @RequestParam Map<String, Object> entity, HttpServletResponse response) {
		entity.remove("daf-auth");
		entity.forEach((key, value) -> {
			if (!"station".equals(key))
			{
				entity.put(key, "%" + value + "%");
			}
		});
		Material vo = DataUtils.mapToEntity(entity, Material::new);
		List<MaterialExcel> list = materialService.exportData(vo);
		response.setContentType("application/vnd.ms-excel");
		response.setCharacterEncoding(Charsets.UTF_8.name());
		String fileName = URLEncoder.encode("物资数据导出", Charsets.UTF_8.name());
		response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
		EasyExcel.write(response.getOutputStream(), MaterialExcel.class).sheet("物资").doWrite(list);
	}

}
